/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2015  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;
const ModalDialog = imports.ui.modalDialog;
const PopupMenu = imports.ui.popupMenu;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    PanelManager - Trash
 */
const KEY_SHOW_TRASH = 'show-trash';

const TRASH_PATH = 'trash:///';
const TRASH_URI = Gio.file_new_for_uri(TRASH_PATH);

const ModalTrashDialog = new Lang.Class({
    Name: 'ModalTrashDialog',
    Extends: ModalDialog.ModalDialog,

    _init: function(subject, description, buttonLabel, callback) {
        this.parent({ styleClass: 'show-processes-dialog' });
        let mainContentBox = new St.BoxLayout({ vertical: false });
        this.contentLayout.add(mainContentBox, { x_fill: true, y_fill: true });
        this.icon = new St.Icon({ icon_name: 'dialog-warning' });
        if (St.IconType) this.icon.icon_type = St.IconType.FULLCOLOR;
        mainContentBox.add(this.icon, { x_fill: true, y_fill: false, x_align: St.Align.END, y_align: St.Align.START });
        let messageBox = new St.BoxLayout({ vertical: true });
        mainContentBox.add(messageBox, { y_align: St.Align.START });
        messageBox.add(new St.Label({ text: subject, style_class: 'show-processes-dialog-subject' }), { y_fill: false, y_align: St.Align.START });
        messageBox.add(new St.Label({ text: description, style_class: 'show-processes-dialog-description' }), { y_fill: true, y_align: St.Align.START });
        this.setButtons([
            { label: _("Cancel"), action: Lang.bind(this, function() { this.close(); }), key: Clutter.Escape },
            { label: _("Open the Trash"), action: Lang.bind(this, function() { this.close(); this._openTrash(); })},
            { label: buttonLabel, action: Lang.bind(this, function() { this.close(); callback(); })}
        ]);
    },

    _openTrash: function() {
        Gio.app_info_launch_default_for_uri(TRASH_PATH, null);
    }
});

const PopupTrashMenuItem = new Lang.Class({
    Name: 'PopupTrashMenuItem',
    Extends: PanelTools.PopupMenuItem,

    _init: function(trashEntry, trashMenu) {
        this.parent(PanelTools.getStringCutOut(trashEntry.get_name()), true);
        this.icon.gicon = trashEntry.get_icon();
        this.addButton('edit-clear-symbolic', Lang.bind(this, function() {
            if (trashMenu.isOpen) trashMenu.close(0);
            Main.overview.hide();
            new ModalTrashDialog(
                PanelTools.getStringLineBreak(_("Are you sure you want to permanently delete '%s' from the trash?").format(PanelTools.getStringCutOut(trashEntry.get_display_name()))),
                _("If you delete an item, it will be permanently lost."),
                _("Delete Permanently"),
                Lang.bind(this, function() {
                    TRASH_URI.get_child(trashEntry.get_name()).delete(null);
                })
            ).open();
        }));
        this.connect('activate', Lang.bind(this, function() {
            Main.overview.hide();
            Gio.app_info_launch_default_for_uri(TRASH_PATH+trashEntry.get_name(), global.create_app_launch_context());
        }));
    }
});

const TrashIndicator = new Lang.Class({
    Name: 'TrashIndicator',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(0.0, _("Trash"), { icon_name: 'user-trash-symbolic' });
        let item = new PanelTools.PopupMenuItem(_("Open the Trash"), true);
        item.setIconName('user-trash-full'); //trashcan_full
        item.connect('activate', Lang.bind(this, function() {
            Main.overview.hide();
            Gio.app_info_launch_default_for_uri(TRASH_PATH, null);
        }));
        this.menu.addMenuItem(item);
        item = new PanelTools.PopupMenuItem(_("Empty Trash..."), true);
        item.setIconName('user-trash'); //trashcan_empty
        item.connect('activate', Lang.bind(this, function() {
            Main.overview.hide();
            new ModalTrashDialog(_("Empty all items from Trash?"),
                _("All items in the Trash will be permanently deleted."),
                _("Empty Trash"),
                Lang.bind(this, function() {
                    let trashChildren = TRASH_URI.enumerate_children('*', 0, null, null);
                    let trashChild = null;
                    while ((trashChild = trashChildren.next_file(null, null)) !== null) {
                        TRASH_URI.get_child(trashChild.get_name()).delete(null);
                    }
                    trashChildren.close(null, null);
                })
            ).open();
        }));
        this.menu.addMenuItem(item);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
//this._trashTimer = false;
        this._trashItems = new PanelTools.PopupSubMenuMenuItem(_("List Trash"), true);
        this._trashItems.setIconName('user-trash-full'); //trashcan_full
        this.menu.addMenuItem(this._trashItems);
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_TRASH, Lang.bind(this, this._updateVisibility));
        this._trashMonitor = TRASH_URI.monitor_directory(0, null, null);
        this._connectTrashMonitor = this._trashMonitor.connect('changed', Lang.bind(this, this._updateVisibility));
        this._updateVisibility();
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._trashMonitor.disconnect(this._connectTrashMonitor);
            this._settings.disconnect(this._connectSettings);
        }));
    },

    _updateVisibility: function() {
        if (this._settings.get_boolean(KEY_SHOW_TRASH)) {
//if (!this._trashTimer && this._settings.get_boolean(KEY_SHOW_TRASH)) {
//this._trashTimer = true;
            this._trashItems.menu.removeAll();
            let trashChild = null;
            let trashChildren = TRASH_URI.enumerate_children('*', 0, null, null);
            let trashEntrys = [];
            while ((trashChild = trashChildren.next_file(null, null)) !== null) {
                trashEntrys.push(trashChild);
            }
            trashChildren.close(null, null);
            if (trashEntrys.length > 0) {
                trashEntrys.sort(function(a, b) {
                    return PanelTools.getSorted(a.get_display_name(), b.get_display_name());
                });
                for (let i = 0; i < trashEntrys.length; i++) {
                    this._trashItems.menu.addMenuItem(new PopupTrashMenuItem(trashEntrys[i], this.menu));
                }
            }
            this.actor.visible = trashEntrys.length > 0;
//this._trashTimer = false;
        } else {
            this.actor.hide();
        }
    }
});

function getExtension() {
    return PanelTools.addToStatusArea('dschaerfTrash',new TrashIndicator(), PanelTools.getChildren('right') - 1, 'right');
}
