/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2015  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;
const Mainloop = imports.mainloop;

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Shell = imports.gi.Shell;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;

/*
    Help Functions
 */
function _activate(menu) {
    if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
        if (menu.isOpen) menu.close(0);
    } else {
        menu.itemActivated();
    }
    Main.overview.hide();
}

function _disabled() {
    if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
        return false;
    } else {
        return Main.sessionMode.isLocked || (Main.sessionMode.isGreeter && this._loginScreenSettings.get_boolean('disable-restart-buttons'));
    }
    return true;
}

/*
    GNOMEShellRestart
 */
function activateGNOMEShellRestart() {
    global.reexec_self();
}

/*
    Hibernate
 */
function _canHibernate(asyncCallback) {
    if (this._proxy) {
        this._proxy.call("CanHibernate", null, Gio.DBusCallFlags.NONE, -1, null, function(proxy, asyncResult) {
            try {
                let result = proxy.call_finish(asyncResult).deep_unpack();
                asyncCallback(result[0] !== 'no');
            } catch(e) {
                asyncCallback(false);
            }
        });
    } else {
        Mainloop.idle_add(Lang.bind(this, function() {
            asyncCallback(false);
        }));
    }
}

function activateHibernate() {
    _activate(this.menu);
    if (Convenience.checkGSV(3, 4)) {
        this._screenSaverProxy.LockRemote(Lang.bind(this, function() {
            this._upClient.hibernate_sync(null);
        }));
    } else if (Convenience.checkGSV(3, 6)) {
        if (this._screenSaverSettings.get_boolean('lock-enabled')) {
            let tmpId = Main.screenShield.connect('lock-screen-shown', Lang.bind(this, function() {
                Main.screenShield.disconnect(tmpId);
                this._upClient.hibernate_sync(null);
            }));
            Main.screenShield.lock(true);
        } else {
            this._upClient.hibernate_sync(null);
        }
    } else {
        if (this._loginManager._proxy) {
            this._loginManager._proxy.call("Hibernate", GLib.Variant.new('(b)', [true]), Gio.DBusCallFlags.NONE, -1, null, null);
        } else {
            this._loginManager.emit('prepare-for-sleep', true);
            this._loginManager.emit('prepare-for-sleep', false);
        }
    }
}

function updateHibernate(actor) {
    if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6)) {
        actor.visible = this._upClient.get_can_hibernate();
    } else {
        _canHibernate.call(this._loginManager, Lang.bind(this, function(result) {
            actor.visible = result && !_disabled();
        }));
    }
}

/*
    LockScreen
 */
function activateLockScreen() {
    _activate(this.menu);
    if (Convenience.checkGSV(3, 4)) {
        this._screenSaverProxy.LockRemote();
    } else {
        Main.screenShield.lock(true);
    }
}

function updateLockScreen(actor) {
    let allowLockScreen = !this._lockdownSettings.get_boolean('disable-lock-screen');
    if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6)) {
        actor.visible = allowLockScreen;
    } else if (Convenience.checkGSV(3, 8)) {
        actor.visible = allowLockScreen && imports.misc.loginManager.canLock();
    } else {
        let showLock = !Main.sessionMode.isLocked && !Main.sessionMode.isGreeter;
        actor.visible = showLock && allowLockScreen && imports.misc.loginManager.canLock();
    }
}

/*
    Logout
 */
function activateLogout() {
    _activate(this.menu);
    this._session.LogoutRemote(0);
}

function updateLogout(actor) {
    let allowLogout = !this._lockdownSettings.get_boolean('disable-log-out');
    if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6)) {
        actor.visible = allowLogout;
    } else if (Convenience.checkGSV(3, 8) || Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12) || Convenience.checkGSV(3, 14) || Convenience.checkGSV(3, 16) || Convenience.checkGSV(3, 18)) {
        let alwaysShow = global.settings.get_boolean('always-show-log-out');
        let systemAccount = this._user.system_account;
        let localAccount = this._user.local_account;
        let multiUser = this._userManager.has_multiple_users;
        let multiSession = imports.gi.Gdm.get_session_ids().length > 1;
        actor.visible = allowLogout && (alwaysShow || multiUser || multiSession || systemAccount || !localAccount);
    }
}

/*
    PanelManagerSettings
 */
function activatePanelManagerSettings() {
    _activate(this.menu);
    Main.Util.spawnCommandLine('gnome-shell-extension-prefs '+Extension.metadata['uuid']);
}

/*
    PowerOff
 */
function activatePowerOff() {
    _activate(this.menu);
    if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6)) {
        this._session.ShutdownRemote();
    } else if (Convenience.checkGSV(3, 8)) {
        this._loginManager.listSessions(Lang.bind(this, function(result) {
            let sessions = [];
            let n = 0;
            for (let i = 0; i < result.length; i++) {
                let[id, uid, userName, seat, sessionPath] = result[i];
                let proxy = new imports.ui.userMenu.SystemdLoginSession(Gio.DBus.system, 'org.freedesktop.login1', sessionPath);
                if (proxy.Class !== 'user') continue;
                if (proxy.State === 'closing') continue;
                if (proxy.Id === GLib.getenv('XDG_SESSION_ID')) continue;
                sessions.push({ user: this._userManager.get_user(userName), username: userName, info: { type: proxy.Type, remote: proxy.Remote }});
                n++;
                if (n === 5) break;
            }
            if (n !== 0) {
                this._openSessionWarnDialog(sessions);
            } else {
                this._session.ShutdownRemote();
            }
        }));
    } else {
        this._session.ShutdownRemote(0);
    }
}

function updatePowerOff(actor) {
    if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6)) {
        this._session.CanShutdownRemote(Lang.bind(this, function(result, error) {
            if (!error) actor.visible = result;
        }));
    } else {
        this._session.CanShutdownRemote(Lang.bind(this, function(result, error) {
            if (!error) actor.visible = result[0];
            actor.visible = result[0] && !_disabled();
        }));
    }
}

/*
    Suspend
 */
function activateSuspend() {
    _activate(this.menu);
    if (Convenience.checkGSV(3, 4)) {
        this._screenSaverProxy.LockRemote(Lang.bind(this, function() {
            this._upClient.suspend_sync(null);
        }));
    } else if (Convenience.checkGSV(3, 6)) {
        if (this._screenSaverSettings.get_boolean('lock-enabled')) {
            let tmpId = Main.screenShield.connect('lock-screen-shown', Lang.bind(this, function() {
                Main.screenShield.disconnect(tmpId);
                this._upClient.suspend_sync(null);
            }));
            Main.screenShield.lock(true);
        } else {
            this._upClient.suspend_sync(null);
        }
    } else {
        this._loginManager.suspend();
    }
}

function updateSuspend(actor) {
    if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6)) {
        actor.visible = this._upClient.get_can_suspend();
    } else if (Convenience.checkGSV(3, 8)) {
        this._loginManager.canSuspend(Lang.bind(this, function(result) {
            actor.visible = result;
        }));
    } else {
        this._loginManager.canSuspend(Lang.bind(this, function(result) {
            actor.visible = result && !disabled;
        }));
    }
}

/*
    SwitchUser
 */
function activateSwitchUser() {
    _activate(this.menu);
    if (Convenience.checkGSV(3, 4)) {
        this._screenSaverProxy.SetActiveRemote(true, Lang.bind(this, function() {
            this._userManager.goto_login_session();
        }));
    } else if (Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
        if (Main.screenShield) Main.screenShield.lock(false);
        imports.gi.Gdm.goto_login_session_sync(null);
    } else {
        if (Main.screenShield) Main.screenShield.lock(false);
        Clutter.threads_add_repaint_func_full(Clutter.RepaintFlags.POST_PAINT, function() {
            imports.gi.Gdm.goto_login_session_sync(null);
            return false;
        });
    }
}

function updateSwitchUser(actor) {
    let allowSwitch = !this._lockdownSettings.get_boolean('disable-user-switching');
    let multiUser = this._userManager.can_switch() && this._userManager.has_multiple_users;
    actor.visible = allowSwitch && multiUser;
}

/*
    Terminal
 */
function activateTerminal() {
    _activate(this.menu);
    Main.Util.spawnCommandLine('gnome-terminal');
}
